# type: ignore[attr-defined]
import threading
import time
from enum import Enum

import typer

from incorta_cloud import version
from incorta_cloud.expection_handlers import create_handler
from incorta_cloud.services import storage, context, auth
from incorta_cloud_client.exceptions import ApiException

from incorta_cloud.utils import logger


class Color(str, Enum):
    white = "white"
    red = "red"
    cyan = "cyan"
    magenta = "magenta"
    yellow = "yellow"
    green = "green"


app = typer.Typer(
    name="incorta-cloud",
    help="Incorta Cloud Client CLI",
    add_completion=False,
)
app.add_typer(storage.storage)
app.add_typer(context.context)
app.add_typer(auth.auth)


def version_callback(print_version: bool) -> None:
    """Print the version of the package."""
    if print_version:
        logger.log(f"[yellow]incorta-cloud[/] version: [bold blue]{version}[/]")
        raise typer.Exit()


def handle_exception(e: Exception):
    handler = create_handler(e)
    if handler is None:
        time.sleep(1)
        raise e
    should_retry = handler.handle(app, e)
    if should_retry:
        app()


def entrypoint():
    with logger as l:
        try:
            app()
        except ApiException as e:
            handle_exception(e)


if __name__ == "__main__":
    entrypoint()
