"""Example of code."""
import time

from rich.progress import Progress, TextColumn, BarColumn, DownloadColumn, TransferSpeedColumn, TimeRemainingColumn


def hello(name: str) -> str:
    """Just an greetings example.

    Args:
        name (str): Name to greet.

    Returns:
        str: greeting message

    Examples:
        .. code:: python

            >>> hello("Roman")
            'Hello Roman!'
    """
    with Progress(TextColumn("[bold blue]{task.fields[filename]}", justify="right"),
                  BarColumn(bar_width=None),
                  "[progress.percentage]{task.percentage:>3.1f}%",
                  "•",
                  DownloadColumn(),
                  "•",
                  TransferSpeedColumn(),
                  "•",
                  TimeRemainingColumn(), ) as progress:

        task1 = progress.add_task("[red]Downloading...", filename= "f1", total=1000, start=False)
        task2 = progress.add_task("[green]Processing...", filename= "f1", total=1000)
        task3 = progress.add_task("[cyan]Cooking...", filename= "f1", total=1000)

        while not progress.finished:
            progress.update(task1, advance=0.5)
            progress.update(task2, advance=0.3)
            progress.update(task3, advance=0.9)
            time.sleep(0.02)
    return f"Hello {name}!"
