import json
import os
from os.path import isfile

from incorta_cloud_client.api_client import ApiClient
from incorta_cloud_client.configuration import Configuration

from incorta_cloud.utils import logger
from incorta_cloud.utils.file_utils import FileUtils

CONTEXT = 'current_context'
CONTEXT_INSTANCE_ID = 'instance_id'
CONTEXT_INSTANCE_NAME = 'instance_name'
CONTEXT_INSTANCE_TENANT_NAME = 'instance_tenant_name'


def get_account_file_path(force_config_file=True):
    path = os.environ.get("INCORTA_CLOUD_CONFIG")
    if path is None:
        logger.error(
            "Please set INCORTA_CLOUD_CONFIG env variable first with the path to the current config file")
        os._exit(1)
    if not isfile(path) and force_config_file:
        logger.error("Incorta Cloud config file not found")
        os._exit(1)
    return path


def parse_account_file(force_config_file=True):
    path = get_account_file_path(force_config_file=force_config_file)
    if not isfile(path) and not force_config_file:
        return dict()
    data = FileUtils.read_data(path)
    json_data = json.loads(data)
    return json_data


def dump_account_file(data: str, force_config_file: bool = True) -> None:
    path = get_account_file_path(force_config_file=force_config_file)
    with open(path, 'w+') as f:
        f.write(data)


def get_incorta_client(force_config_file=True):
    account_conf = parse_account_file(force_config_file=force_config_file)
    tok = ""
    if account_conf:
        tok = account_conf["token"]
    conf = Configuration(
        # host="https://staging-cloud-upload-server-vncvkzqu7a-uc.a.run.app/api/v1",
        host="https://production-cloud-upload-server-ahjisf5osa-uc.a.run.app/api/v1",
        # host="https://clouddev-mc.incortacloud-dev.com/api/v1",
        # host="https://clouddev-cloud-upload-server-vncvkzqu7a-uc.a.run.app/api/v1",
        # host="https://clouddev-cloud-upload-server-vncvkzqu7a-uc.a.run.app/api/v1",
        # host="http://localhost:8080/api/v1",
        api_key={
            "ApiKeyAuth": tok
        }
    )
    return ApiClient(configuration=conf)


def set_incorta_context(instance_id, instance_name, tenant_name):
    account_conf = parse_account_file()
    ctx = dict()
    ctx[CONTEXT_INSTANCE_ID] = instance_id
    ctx[CONTEXT_INSTANCE_NAME] = instance_name
    ctx[CONTEXT_INSTANCE_TENANT_NAME] = tenant_name
    account_conf[CONTEXT] = ctx
    dump_account_file(json.dumps(account_conf))


def get_current_context():
    account_conf = parse_account_file()

    if CONTEXT not in account_conf:
        return None

    ctx = account_conf[CONTEXT]

    if CONTEXT_INSTANCE_ID not in ctx or CONTEXT_INSTANCE_NAME not in ctx:
        return None

    return ctx[CONTEXT_INSTANCE_ID], ctx[CONTEXT_INSTANCE_NAME], ctx[CONTEXT_INSTANCE_TENANT_NAME]


def set_user_token(token):
    account_conf = parse_account_file(force_config_file=False)
    account_conf['token'] = token
    dump_account_file(json.dumps(account_conf, ), force_config_file=False)
