import typer
from incorta_cloud_client import ApiClient
from incorta_cloud_client.exceptions import UnauthorizedException
from incorta_cloud_client.model.forms_user_sign_in_request import FormsUserSignInRequest
from incorta_cloud_client.api.user_api import UserApi
from rich.console import Console
from typer import Typer

from incorta_cloud.incorta_client import get_incorta_client, set_user_token
from incorta_cloud.utils import logger


class SignIn:

    def __init__(self, app: Typer, console: Console) -> None:
        super().__init__()
        self.app = app
        self.console = console

        @app.command()
        def signin(email: str = typer.Option(..., prompt=True), password: str = typer.Option(..., prompt=True, hide_input=True)) -> bool:
            client = get_incorta_client(force_config_file=False)

            return self.signin(email, password, client)

    def signin(self, email: str, password: str, client: ApiClient) -> bool:
        with client as api_client:
            api_instance = UserApi(api_client)

            data = FormsUserSignInRequest(email=email, password=password)
            try:
                resp = api_instance.sign_in(data)
            except UnauthorizedException as e:
                logger.error("Invalid Credentials")
                return False
            set_user_token(resp.payload.token)
            logger.success("Logged in Successfully!")
            return True
