import rich
import typer
from incorta_cloud_client.api.user_api import UserApi
from typer import Typer
from rich.console import Console
from incorta_cloud_client.api_client import ApiClient
from incorta_cloud_client.exceptions import ApiException

from ...incorta_client import get_incorta_client, set_incorta_context
from ...utils import logger


class SetContext:

    def __init__(self, app: Typer, console: Console) -> None:
        super().__init__()
        self.app = app
        self.console = console

        @app.command()
        def set_context(instance_name: str = typer.Option(...), tenant_name: str = typer.Option(...)) -> None:
            client = get_incorta_client()

            self.set_context(instance_name, tenant_name, client)

    def set_context(self, cluster_name: str, tenant_name: str, client: ApiClient) -> None:
        with client as api_client:
            api_instance = UserApi(api_client)
            instances = api_instance.get_instances()
            for instance in instances.payload:
                if instance.instance.name != cluster_name:
                    continue
                try:
                    tenants = api_instance.get_instance_tenants(instance.instance.name)
                    for tenant in tenants.payload:
                        if tenant.name != tenant_name:
                            continue
                        set_incorta_context(instance.instance.id, instance.instance.name, tenant.name)
                        logger.success("Context set successfully")
                        return
                    logger.error("Invalid Tenant name. Tenant doesn't exist on this cluster")
                    return
                except ApiException as e:
                    if e.status == 400:
                        logger.error("This cluster is being created and can not verify tenant name.\nPlease wait until creation completion and try again.")
                        return
                    raise e

            logger.error("Couldn't find cluster under user resources")
