import os.path
import threading

import requests
import urllib3
from incorta_cloud_client.model.models_download_metadata import ModelsDownloadMetadata
from incorta_cloud.utils import logger, string_utils
from incorta_cloud.utils.file_utils import FileUtils
from incorta_cloud.utils.logger.logger import Logger

http = urllib3.PoolManager()


class DownloadWorker(threading.Thread):
    def __init__(self, root_dir_path: str, file_metadata: ModelsDownloadMetadata) -> None:
        # threading.Thread.__init__(self)
        super().__init__()
        self.file_metadata = file_metadata
        self.root_dir_path = root_dir_path

    def run(self) -> None:
        self.download()

    @staticmethod
    def file_head(url: str):
        r = http.request('HEAD', url)
        return {
            'length': int(r.headers['x-goog-stored-content-length']),
            'hash': r.headers['x-goog-hash']
        }

    def download(self):  # pass

        progresObj = logger.url_progress(Logger.download_progress_name, "red")

        file_head = DownloadWorker.file_head(self.file_metadata.download_url)
        dir_path = os.path.join(os.path.abspath(
            self.root_dir_path), string_utils.remove_leading_slash(self.file_metadata.path))
        download_path = os.path.join(
            dir_path, string_utils.remove_leading_trailing_slash(self.file_metadata.name))

        task_id = logger.add_task_to_progress(Logger.download_progress_name, self.file_metadata.download_url, file_head['length'],
                                                   self.file_metadata.path + self.file_metadata.name, "Downloading")
        response = requests.get(self.file_metadata.download_url, stream=True)

        # logger.debug(download_path)
        dl = 0
        FileUtils.create_dirs(dir_path)
        # logger.debug(response.content)
        # logger.debug(self.file_metadata.download_url)
        with open(download_path, 'wb+') as f:
            for data in response.iter_content(chunk_size=4096):
                dl += len(data)
                logger.update_task_progress(
                    Logger.download_progress_name, task_id, dl)
                f.write(data)
            logger.update_task_progress(
                Logger.download_progress_name, task_id, file_head['length'])
