import os

import typer
from incorta_cloud_client import ApiClient
from typer import Typer

from incorta_cloud.utils.logger.logger import Logger

from .upload_utils import upload_data, initiate_file_upload
from ...incorta_client import get_incorta_client
from ...utils.file_utils import FileUtils
from ...utils.string_utils import parse_name

from ...utils import logger


class UploadFile:

    def __init__(self, app: Typer) -> None:
        super().__init__()
        self.app = app

        @app.command()
        def upload(local_path: str = typer.Option(..., exists=True, file_okay=True, dir_okay=False, readable=True),
                   target_path: str = typer.Option(..., exists=True, file_okay=True, dir_okay=False, readable=True)) -> None:
            client = get_incorta_client()
            self.upload(logger, local_path, target_path, client)

    def upload(self, logger: Logger, path: str, target_path: str, client: ApiClient, upload_url: str = None) -> None:
        path = os.path.abspath(path)
        _, name = parse_name(path, None)
        data = FileUtils.read_data(path)

        if upload_url is None:
            payload = initiate_file_upload(
                name, path, client, target_path=target_path)
            if payload is None:
                ## TODO handle None in upload itself
                return
            upload_url = payload.upload_url

        upload_data(logger, data, upload_url,
                    os.path.join(target_path, name))
