import threading

import typer

from incorta_cloud.services.storage.upload_file import UploadFile
from typer import Typer

from ...incorta_client import get_incorta_client

from ...utils.concurrency.concurrency_manager import ConcurrencyManager
from ...utils.file_utils import FileUtils
from ...utils import logger


class UploadTenant:

    def __init__(self, app: Typer) -> None:
        super().__init__()
        self.app = app

        @app.command()
        def upload_tenant(root_path: str = typer.Option(..., exists=True, file_okay=False, dir_okay=True), parallel_uploads: int = typer.Option(10, min=1, max=20)) -> None:
            uploader = UploadFile(app)
            client = get_incorta_client()

            file_generator = FileUtils.dir_files_generator(root_path)
            all_files = list(file_generator)

            thread_generator = (UploadWorker(uploader, client, abs_path, rel_path, file_name) for (
                abs_path, rel_path, file_name) in all_files)

            ConcurrencyManager.execute_in_parallel(
                thread_generator, parallel_uploads)


class UploadWorker(threading.Thread):
    def __init__(self, uploader: UploadFile, client, abs_path, rel_path, file_name, upload_url: str = None):
        threading.Thread.__init__(self)
        self.upload_url = upload_url
        self.uploader = uploader
        self.client = client
        self.absPath = abs_path
        self.relPath = rel_path
        self.file_name = file_name
        self.logger = logger

    def run(self):
        self.uploader.upload(self.logger, self.absPath, self.relPath,
                             self.client, self.upload_url)
